--
-- $Id: HrefDatabase.hs,v 1.3 2005/12/03 19:17:24 aamine Exp $
--
-- Copyright (c) 2003-2005 Minero Aoki
--
-- This program is free software.
-- You can distribute/modify this program under the terms of
-- the GNU LGPL, Lesser General Public License version 2.1.
-- For details of the GNU LPGL, see the file "COPYING".
--

module HrefDatabase
    (Entry (..), entContent, EntrySpec (..), specMatchBy,
     putEntry, putIndex,
     readModules, readSpecs, readModuleFunctions, readDocument,
     hrefDataDirEnv)
where

import PathUtils
import Utils

import Data.Char (isAlphaNum)
import Data.List
import Data.Maybe
import Network.URI (escapeURIString, unEscapeString)

hrefDataDirEnv = "HREF_DATADIR"
hrefKeyListFile = "comptable"

data Entry = Entry EntrySpec String

entContent (Entry s c) = c

entPath prefix (EntrySpec m f) = joinPath [prefix, encodePath m, encodePath f]

data EntrySpec = EntrySpec String String

specModule (EntrySpec m f) = m

specFunction (EntrySpec m f) = f

instance Show EntrySpec where
  show (EntrySpec m f) = m ++ "." ++ escapeFunction f

escapeFunction s
    | isJust $ find (not . isAlphaNum) s  = "(" ++ s ++ ")"
    | otherwise = s

serializeSpec (EntrySpec m f) = m ++ "," ++ f

parseSpec s = let (m,f) = break (== ',') s
              in EntrySpec m (tail f)

specMatchBy :: String -> String -> (String -> String -> Bool) -> EntrySpec -> Bool
specMatchBy ""   ""   f _                    = True
specMatchBy ""   fpat f (EntrySpec _ func)   = f fpat func
specMatchBy mpat ""   f (EntrySpec mod _)    = f mpat mod
specMatchBy mpat fpat f (EntrySpec mod func) = f mpat mod && f fpat func

putEntry :: FilePath -> Entry -> IO ()
putEntry prefix (Entry s c) =
    do makePath (dirname $ entPath prefix s)
       writeFile (entPath prefix s) $ c

putIndex :: FilePath -> [Entry] -> IO ()
putIndex prefix ents =
    do makePath prefix
       writeFile (comptablePath prefix) $ unlines $ map (entIndex) ents
  where
    comptablePath prefix = joinPath [prefix, hrefKeyListFile]

    entIndex (Entry s c) = serializeSpec s

readModules prefix =
    return . map (decodePath) . filter (elem '%') =<< dirEntries prefix

readSpecs :: FilePath -> IO [EntrySpec]
readSpecs prefix =
    return . map (parseSpec) . lines
    =<< readFile (joinPath [prefix, hrefKeyListFile])

readModuleFunctions :: FilePath -> String -> IO [String]
readModuleFunctions prefix mod =
    return . map (decodePath) =<< dirEntries (joinPath [prefix, encodePath mod])

readDocument :: FilePath -> EntrySpec -> IO String
readDocument prefix spec = readFile (entPath prefix spec)

encodePath = escapeURIString (\c -> isAlphaNum c || c == '_')

decodePath = unEscapeString
